@@ -1,33 +1,32 @@
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2002, 2004  Free Software Foundation, Inc.
+ *  Copyright (C) 2002,2004,2006,2007  Free Software Foundation, Inc.
  *
- *  GRUB is free software; you can redistribute it and/or modify
+ *  GRUB is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  GRUB is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with GRUB; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef GRUB_TYPES_CPU_HEADER
 #define GRUB_TYPES_CPU_HEADER	1
 
 /* The size of void *.  */
-#define GRUB_HOST_SIZEOF_VOID_P	8
+#define GRUB_TARGET_SIZEOF_VOID_P	8
 
 /* The size of long.  */
-#define GRUB_HOST_SIZEOF_LONG	8
+#define GRUB_TARGET_SIZEOF_LONG		8
 
 /* sparc64 is big-endian.  */
-#define GRUB_HOST_WORDS_BIGENDIAN	1
+#define GRUB_TARGET_WORDS_BIGENDIAN	1
 
 
 #endif /* ! GRUB_TYPES_CPU_HEADER */
